@@ -289,6 +289,49 @@ static void create_devices(vps_handler *h, envid_t veid, const char *root)
 	}
 }
 
+static int mount_devpts(void)
+{
+	int fd, ret;
+
+	setuid(0);
+	setgid(0);
+
+	ret = mkdir("/dev/pts", S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
+	if ((ret < 0) && (errno != EEXIST)) {
+		logger(-1, errno, "Cannot create container's /dev/pts");
+		return VZ_RESOURCE_ERROR;
+	}
+	ret = mount("devpts", "/dev/pts", "devpts", 0, "newinstance");
+	if (ret < 0) {
+		/* No need to cleanup mkdir, since we test for EEXIST */
+		logger(-1, errno, "Cannot mount container's /dev/pts");
+		return VZ_RESOURCE_ERROR;
+	}
+
+	/* /dev/ptmx, if it even exists, would refer to the root ptmx.
+	 * We don't want that, we want our newly created instance to contain
+	 * all ptys. So we bind mount the root device here
+	 */
+	fd = open("/dev/ptmx", O_CREAT, S_IRWXU|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
+	if (fd < 0) {
+		logger(-1, errno, "Cannot create container's /dev/ptmx");
+		/*
+		 * No need to umount, we are in a private mnt namespace and it will
+		 * disappear after we fail.
+		 */
+		return VZ_RESOURCE_ERROR;
+	}
+	close(fd);
+	ret = mount("/dev/pts/ptmx", "/dev/ptmx", "", MS_BIND, 0);
+	if (ret < 0) {
+		/* No need to cleanup mkdir, since we test for EEXIST */
+		logger(-1, errno, "Cannot bind mount container's /dev/ptmx");
+		return VZ_RESOURCE_ERROR;
+	}
+
+	return 0;
+}
+
 static int _env_create(void *data)
 {
 	struct arg_start *arg = data;
@@ -322,49 +365,9 @@ static int _env_create(void *data)
 	if (ret)
 		return ret;
 
-	if (arg->h->can_join_userns) {
-		int fd, ret;
-		setuid(0);
-		setgid(0);
-		/*
-		 * We need the special flag "newinstance". This is a requirement
-		 * of the userns-aware implementation of devpts as of Linux 3.9.
-		 * Because of that special requirement, we do it here rather than
-		 * later.
-		 */
-		ret = mkdir("/dev/pts", S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
-		if ((ret < 0) && (errno != EEXIST)) {
-			logger(-1, errno, "Cannot create container's /dev/pts");
-			return VZ_RESOURCE_ERROR;
-		}
-		ret = mount("devpts", "/dev/pts", "devpts", 0, "newinstance");
-		if (ret < 0) {
-			/* No need to cleanup mkdir, since we test for EEXIST */
-			logger(-1, errno, "Cannot mount container's /dev/pts");
-			return VZ_RESOURCE_ERROR;
-		}
-
-		/* /dev/ptmx, if it even exists, would refer to the root ptmx.
-		 * We don't want that, we want our newly created instance to contain
-		 * all ptys. So we bind mount the root device here
-		 */
-		fd = open("/dev/ptmx", O_CREAT, S_IRWXU|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
-		if (fd < 0) {
-			logger(-1, errno, "Cannot create container's /dev/ptmx");
-			/*
-			 * No need to umount, we are in a private mnt namespace and it will
-			 * disappear after we fail.
-			 */
-			return VZ_RESOURCE_ERROR;
-		}
-		close(fd);
-		ret = mount("/dev/pts/ptmx", "/dev/ptmx", "", MS_BIND, 0);
-		if (ret < 0) {
-			/* No need to cleanup mkdir, since we test for EEXIST */
-			logger(-1, errno, "Cannot bind mount container's /dev/ptmx");
-			return VZ_RESOURCE_ERROR;
-		}
-	}
+	ret = mount_devpts();
+	if (ret)
+		return ret;
 
 	/*
 	 * If we are using the user namespace, we will have the full capability
@@ -999,8 +1002,6 @@ static int ct_restore(vps_handler *h, envid_t veid, vps_param *vps_p, int cmd,
 int ct_do_open(vps_handler *h, vps_param *param)
 {
 	int ret;
-	char path[STR_SIZE];
-	char upath[STR_SIZE];
 	struct stat st;
 	unsigned long *local_uid = param->res.misc.local_uid;
 
@@ -1015,12 +1016,6 @@ int ct_do_open(vps_handler *h, vps_param *param)
 		return VZ_RESOURCE_ERROR;
 	}
 
-	if (snprintf(path, sizeof(path), "/proc/%d/ns/pid", getpid()) < 0)
-		return VZ_RESOURCE_ERROR;
-
-	if (snprintf(upath, sizeof(upath), "/proc/%d/ns/user", getpid()) < 0)
-		return VZ_RESOURCE_ERROR;
-
 	ret = mkdir(NETNS_RUN_DIR, S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
 
 	if (ret && (errno != EEXIST)) {
@@ -1029,7 +1024,7 @@ int ct_do_open(vps_handler *h, vps_param *param)
 		return VZ_RESOURCE_ERROR;
 	}
 
-	h->can_join_pidns = !stat(path, &st);
+	h->can_join_pidns = !stat("/proc/self/ns/pid", &st);
 	/*
 	 * Being able to join the user namespace is a good indication that the
 	 * user namespace is complete. For a long time, the user namespace
@@ -1043,7 +1038,8 @@ int ct_do_open(vps_handler *h, vps_param *param)
 	 * mapped user to own the files, etc. So we also need to find suitable
 	 * configuration in the config files.
 	 */
-	h->can_join_userns = !stat(upath, &st) && local_uid && (*local_uid != 0);
+	h->can_join_userns = !stat("/proc/self/ns/user", &st) &&
+				local_uid && (*local_uid != 0);
 	h->is_run = ct_is_run;
 	h->enter = ct_enter;
 	h->destroy = ct_destroy;
